<?php
// $Id: regtag_helper.inc,v 1.13 2006/10/18 09:59:37 profix898 Exp $

/**
 * function _reptag_table_load().
 * (load tags from table)
 */
function _reptag_table_load($iid, $cleanlevel = 0) {
  // build query
  $query = 'SELECT * FROM {reptag_vars} WHERE uid = \''.intval($iid[0]).'\'';
  for ($i = 1; $i < count($iid); $i++) {
    $query .= ' OR uid = \''.intval($iid[$i]).'\'';
  }
  if (variable_get('reptag_tableprio', 0)) {
    $query .= ' ORDER BY uid';
  }
  else {
    $query .= ' ORDER BY uid DESC';
  }
  // load and format tags
  $vars = array();
  $result = db_query($query);
  while ($var = db_fetch_object($result)) {
    switch ($cleanlevel) {
    case 1:
      $vars['$'.substr($var->name, 3, -5).'$'] = unserialize($var->value);
      break;
    case 2:
      $vars[substr($var->name, 3, -5)] = unserialize($var->value);
      break;
    case 0:
    default:
      $vars[$var->name] = unserialize($var->value);
    }
  }
   
  return $vars;
}

/**
 * function _reptag_table_set().
 * (store tag to table)
 */
function _reptag_table_set($name, $value, $uid) {
  db_lock_table('reptag_vars');
  db_query("DELETE FROM {reptag_vars} WHERE name = '%s' AND uid = '%d'", $name, $uid);
  db_query("INSERT INTO {reptag_vars} (name, value, uid) VALUES ('%s', '%s', '%d')", $name, serialize($value), $uid);
  db_unlock_tables();
}

/**
 * function _reptag_table_del().
 * (remove tag from table)
 */
function _reptag_table_del($name, $uid) {
  db_lock_table('reptag_vars');
  db_query("DELETE FROM {reptag_vars} WHERE name = '%s' AND uid = '%d'", $name, $uid);
  db_unlock_tables();
}

/**
 * function _reptag_table_clear().
 * (remove all tags for a specified user)
 */
function _reptag_table_clear($uid) {
  db_lock_table('reptag_vars');
  db_query("DELETE FROM {reptag_vars} WHERE uid = '%d'", $uid);
  db_unlock_tables();
}

/**
 * function _reptag_roles_load().
 * (load all available roles)
 */
function _reptag_roles_load() {
  $roles = array();
  $result = db_query("SELECT * FROM {role}");
  while ($var = db_fetch_object($result)) {
    $roles[$var->name] = $var->rid;
  }
  
  return $roles;
}

/**
 * function _reptag_tags_implode().
 */
function _reptag_tags_implode($tags) {
  $tags = array_filter($tags);
  $tags = implode('|', $tags);
  return $tags;
}

/**
 * function _reptag_exclude_loadtags().
 * (load exclude tags from variable)
 */
function _reptag_exclude_loadtags($rid) {
  $tags = unserialize(variable_get('reptag_exclude_r'.$rid, serialize(array())));
  return $tags;
}

/**
 * function _reptag_exclude_loadtags_string().
 * (load exclude tags from variable into a string)
 */
function _reptag_exclude_loadtags_string($rid) {
  $tags = _reptag_exclude_loadtags($rid);
  $tags = _reptag_tags_implode($tags);
  return $tags;
}

/**
 * function _reptag_exclude_storetags().
 * (store exclude tags to variable)
 */
function _reptag_exclude_storetags($rid, $tags) {
  variable_set('reptag_exclude_r'.$rid, serialize($tags));
}

/**
 * function _reptag_exclude_storetags_string().
 * (explode string and store exclude tags to variable)
 */
function _reptag_exclude_storetags_string($rid, $tags) {
  $tags = explode('|', $tags);
  _reptag_exclude_storetags($rid, $tags);
}

/**
 * function _reptag_user_get_roles().
 * (get roles for a specified user)
 */
function _reptag_user_get_roles($uid) {
  $roles = array();
  switch ($uid) {
  case 0: //anonymous user
    $roles[] = '1';
  	break;
  case 1: //administrator
    $roles[] = '1';
    $roles[] = '2';  
    $result = db_query("SELECT DISTINCT rid FROM {users_roles}");
    while ($rid = db_fetch_object($result)) {
      $roles[] = $rid->rid;
    }	
  	break;
  default: //authenticated user  	
    $roles[] = '2';
    $result = db_query("SELECT rid FROM {users_roles} WHERE uid = '%d'", $uid);
    while ($rid = db_fetch_object($result)) {
      $roles[] = $rid->rid;
    }
  	break;
  }
  
  return $roles;
}

/**
 * function _reptag_node_update().
 * (update body/teaser for a specified node)
 */
function _reptag_node_update($node) {
  db_lock_table('node_revisions');
  db_query("UPDATE {node_revisions} SET body = '%s', teaser = '%s' WHERE vid = %d",
    $node->body, $node->teaser, $node->vid);
  db_unlock_tables();
  cache_clear_all();
}

/**
 * function _reptag_storage_setpage().
 * (save body/teaser of a node to storage table)
 */
function _reptag_storage_setpage($node) {
  $result = db_query("SELECT nid FROM {reptag_storage} WHERE vid = '%d'", $node->vid);
  if (db_num_rows($result)) {
    db_lock_table('reptag_storage');
    db_query("UPDATE {reptag_storage} SET body = '%s', teaser = '%s' WHERE vid = %d",
      $node->body, $node->teaser, $node->vid);
    db_unlock_tables();
  }
  else {
    _reptag_storage_addpage($node);
  }
}

/**
 * function _reptag_storage_addpage().
 * (store body/teaser into storage table)
 */
function _reptag_storage_addpage($node) {
  db_query("INSERT INTO {reptag_storage} (nid, vid, body, teaser) VALUES ('%d', '%d', '%s', '%s')",
    $node->nid, $node->vid, $node->body, $node->teaser);
}

/**
 * function _reptag_storage_removepage().
 * (remove a node from storage table)
 */
function _reptag_storage_removepage($node) {
  db_lock_table('reptag_storage');
  db_query("DELETE FROM {reptag_storage} WHERE nid = '%d'", $node->nid);
  db_unlock_tables();
}

/**
 * function _reptag_storage_loadpage().
 * (load page content (body/teaser) from reptag storage)
 */
function _reptag_storage_loadpage($vid) {
  if (is_array($vid)) {
    $vid = $vid->vid;
  }
  $result = db_query("SELECT * FROM {reptag_storage} WHERE vid = '%d'", $vid);
  
  return db_fetch_object($result);
}

/**
 * function _reptag_match_tags().
 * (filter $tags containing $string)
 */
function _reptag_match_tags($string, $tags) {
  $matches = array();
  foreach ($tags as $tag) {
    if (!(strpos($tag, $string) === false)) {
      $matches[$tag] = check_plain($tag);
    }
  }
  
  return $matches;
}

?>
