<?php

/**
 * function _reptag_manage_admin_form().
 * (forms for administration pages: general settings)
 */
function _reptag_manage_admin_general_form() {
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General - Settings'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE
  );
  $form['general']['reptag_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Rep[lacement]Tags'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_enable', 0)
  );
  $form['general']['reptag_preview'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Rep[lacement]Tags in Preview'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_preview', 1)
  );
  $form['general']['reptag_showhelp'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show \'Rep[lacement]Tags - Help\' section on add/edit node pages'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_showhelp', 1)
  );
  $form['general']['reptag_tableprio'] = array(
    '#type' => 'checkbox',
    '#title' => t('Privilege SiteWide tags over User tags'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_tableprio', 0),
    '#description' => t('By default users can override site wide tags with their own tags.')
  );
  
  // node-types
  $node_types = node_get_types('names');
  if (!empty($node_types)) {
    $form['general']['node_types'] = array(
      '#type' => 'fieldset',
      '#title' => t('NodeType - Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
      '#weight' => '1'
    );
    foreach ($node_types as $type => $title) {
      $form['general']['node_types'][$type] = array(
        '#type' => 'checkbox',
        '#title' => t('Disable RepTags for !module', array('!module' => '<strong>'.$title.'</strong>')),
        '#return_value' => 1,
        '#default_value' => variable_get('reptag_type_disable_'.$type, 0)
      );
    }
    if (module_exists('comment')) {
      $form['general']['node_types']['comments'] = array(
        '#type' => 'checkbox',
        '#title' => t('Disable RepTags for !module', array('!module' => '<strong>'.t('Comments').'</strong>')),
        '#return_value' => 1,
        '#default_value' => variable_get('reptag_type_disable_comments', 1)
      );
    }
  }
  
  // expert - settings
  $form['general']['expert'] = array(
    '#type' => 'fieldset',
    '#title' => t('Expert - Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => '2'
  );  
  $form['general']['expert']['reptag_validate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Preview - Validation'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_validate', 1),
    '#description' => t('Run html-filter on Preview to find invalid tags.')
  );
  $form['general']['expert']['reptag_cache'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Partial PageCache'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_cache', 0),
    '#description' => t('<i>(recommended at least for high traffic sites)</i><br />Expand static tags on form submission and cache the result. Only dynamic tags are processed on page view.<br />To disable dynamic tags completely check \'Disable Dynamic Replacements\' under \'Expert-Settings\'.')
  );
  $form['general']['expert']['reptag_staticonly'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable Dynamic Replacements'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_staticonly', 0),
    '#description' => t('Run reptag on form submission only (disables certain modules, e.g. System, Block, ...).<br />Modules affected by this option are marked \'Static\' = \'No\' below.'),
    '#disabled' => variable_get('reptag_cache', 0) ? FALSE : TRUE
  );
  $form['general']['expert']['reptag_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Debugging'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_debug', 0),
    '#description' => t('Print debug listings (print_r) of module variables.')
  );
  // user interface
  $form['general']['ui'] = array(
    '#type' => 'fieldset',
    '#title' => t('UserInterface - Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => '3'
  );
  $form['general']['ui']['reptag_add_sample'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add sample data to tag tables'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_add_sample', 1),
    '#description' => t('Adds a row of sample data to site-/user-tag tables.')
  );
  $form['general']['ui']['reptag_javascript'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Javascript/AJAX - Support'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_javascript', 1),
    '#description' => t('Enable Javascript/AJAX - Support for administration pages.')
  );
  $form['general']['ui']['reptag_textarea'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use textareas for replacements'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_textarea', 1),
    '#description' => t('Use resizable textareas instead of textfields for replacements in tags table.')
  );
  // integration
  $form['general']['integration'] = array(
    '#type' => 'fieldset',
    '#title' => t('Integration - Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => '4'
  );
  $form['general']['integration']['reptag_quicktags'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Quicktags - Integration'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_quicktags', 0),
    '#description' => t('Add a button to Quicktag\'s toolbar. Available only with \'Quicktags\' module installed.'),
    //'#disabled' => module_exists('quicktags') ? FALSE : TRUE
    '#disabled' => TRUE // not available atm
  );
  $form['general']['integration']['reptag_workspace'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Workspace - Integration'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_workspace', 0),
    '#description' => t('Adds a tab to the user\'s workspace. Available only with \'Workspace\' module installed.'),
    '#disabled' => module_exists('workspace') ? FALSE : TRUE
  );
  
  $form['#base'] = '_reptag_manage_admin_general';
  return _reptag_manage_form($form);
}

/**
 * function _reptag_manage_admin_modules_form().
 * (forms for administration pages: module settings)
 */
function _reptag_manage_admin_modules_form() {
  $form['modules'] = array(
    '#type' => 'fieldset',
    '#title' => t('Modules'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );
  $files = file_scan_directory(drupal_get_path('module', 'reptag'), '\.tags$');
  $form['modules']['table'] = array('#theme' => 'reptag_modules_table');
  foreach ($files as $filename => $file) {
    include_once $file->filename;
    $static_module = is_array($reptags_info[$file->name]) ? $reptags_info[$file->name][1] : FALSE;
    $disable_module = variable_get('reptag_cache', 0) && variable_get('reptag_staticonly', 0) && !$static_module;
    $form['modules']['table'][$file->name]['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => '',
      '#return_value' => 1,
      '#default_value' => $disable_module ? FALSE : variable_get('reptag_module_'.$file->name, 0),
      '#prefix' => '<div align="center">',
      '#suffix' => '</div>',
      '#disabled' => $disable_module ? TRUE : FALSE
    );
    $form['modules']['table'][$file->name]['module'] = array(
      '#value' => $file->basename,
      '#prefix' => '<strong>',
      '#suffix' => '</strong>'
    );
    $form['modules']['table'][$file->name]['info'] = array(
      '#value' => is_array($reptags_info[$file->name]) ?
        $reptags_info[$file->name][0] : $reptags_info[$file->name],
      '#prefix' => '<i>',
      '#suffix' => '</i>'
    );
    $form['modules']['table'][$file->name]['static'] = array(
      '#value' => $static_module ? t('Yes') : t('No'),
      '#prefix' => '<div align="center">',
      '#suffix' => '</div>'
    );
  }
  
  $form['#base'] = '_reptag_manage_admin_modules';
  return _reptag_manage_form($form);
}

/**
 * function _reptag_manage_admin_roles_form().
 * (forms for administration pages: module settings)
 */
function _reptag_manage_admin_roles_form() {
  $form['roles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Roles / Permissions'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );
  
  // module permissions
  $roles = _reptag_roles_load();
  $files = array_values(file_scan_directory(drupal_get_path('module', 'reptag'), '\.tags$'));
  $form['roles']['table'] = array('#theme' => 'reptag_roles_table');
  $form['roles']['table']['header'] = array('#data' => array_merge(array(t('Option / Module')), array_keys($roles)));
  foreach($files as $file) {
    $module_enabled = variable_get('reptag_module_'.$file->name, 0);
    $form['roles']['table'][$file->name]['label'] = array(
      '#value' => t('Module: !module', array('!module' => '<strong>'.$file->basename.'</strong>')),
    );
    foreach ($roles as $rid) {
      $form['roles']['table'][$file->name][$rid] = array(
        '#type' => 'checkbox',
        '#title' => '',
        '#return_value' => 1,
        '#default_value' => in_array($rid, (array)unserialize(variable_get('reptag_module_'.$file->name.'_roles', ''))),
        '#prefix' => '<div align="center">',
        '#suffix' => '</div>',
        '#disabled' => $module_enabled ? FALSE : TRUE
      );
    }    
  }
  // spacer
  $form['roles']['table']['spacer']['label'] = array('#value' => '');
  foreach ($roles as $rid) {
    $form['roles']['table']['spacer'][$rid] = array('#value' => '');
  }
  // additional permissions
  $form['roles']['table']['plainrep']['label'] = array(
    '#value' => t('Plain text replacements only'),
    '#prefix' => '<strong>',
    '#suffix' => '</strong>'
  );
  foreach ($roles as $rid) {
    $form['roles']['table']['plainrep'][$rid] = array(
      '#type' => 'checkbox',
      '#title' => '',
      '#return_value' => 1,
      '#default_value' => in_array($rid, (array)unserialize(variable_get('reptag_module_plainrep_roles', ''))),
      '#prefix' => '<div align="center">',
      '#suffix' => '</div>'
    );
  }
  
  // exclude tags
  $form['roles']['exclude'] = array(
    '#type' => 'fieldset',
    '#title' => t('Exclude Tags'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE,
    '#prefix' => '<br />'
  );
  $form['roles']['exclude']['table'] = array('#theme' => 'reptag_exclude'); 
  foreach ($roles as $role => $rid) {
    $form['roles']['exclude']['table'][$rid]['role'] = array(
      '#value' => $role,
      '#prefix' => '<strong>',
      '#suffix' => '</strong>'
    );
    $form['roles']['exclude']['table'][$rid]['tags'] = array(
      '#type' => 'textfield',
      '#default_value' => _reptag_exclude_loadtags_string($rid),
      '#size' => 60,
      '#maxlength' => 1024
    );
    $form['roles']['exclude']['table'][$rid]['append'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Append'),
      '#name' => 'append',
      '#attributes' => array('id' => $rid),
      '#button_type' => 'reptag-button-append',
    );
    $form['roles']['exclude']['table'][$rid]['remove'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Remove'),
      '#name' => 'remove',
      '#attributes' => array('id' => $rid),
      '#button_type' => 'reptag-button-remove',
    );
  }
  if (variable_get('reptag_javascript', 1)) {
    $form['roles']['exclude']['container']['#parents'] = array();
    $form['roles']['exclude']['container']['add'] = array('#theme' => 'reptag_exclude_add'); 
    $form['roles']['exclude']['container']['add']['value'] = array(
      '#type' => 'textfield',
      '#title' => '',
      '#default_value' => '',
      '#size' => 35,
      '#maxlength' => 128,
      '#autocomplete_path' => 'reptag/autocomplete'
    );
    $form['roles']['exclude']['container']['add']['button'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Add'),
      '#name' => 'add',
      '#button_type' => 'reptag-button-add'
    );
  }
  
  $form['#base'] = '_reptag_manage_admin_roles';
  return _reptag_manage_form($form);
}

/**
 * function _reptag_manage_site_form().
 * (forms for tags administration: site-wide tags)
 */
function _reptag_manage_site_form() {
  $form['site'] = array(
    '#type' => 'fieldset',
    '#title' => t('SiteWide - Rep[lacement]Tags'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );
  
  $form['site'][] = _reptag_manage_table_builder();
  $form['#base'] = '_reptag_manage_tags';
  return _reptag_manage_form($form);
}

/**
 * function _reptag_manage_user_form().
 * (forms for tags administration: user tags)
 */
function _reptag_manage_user_form() {
  global $user;  
  $form['user'] = array(
    '#type' => 'fieldset',
    '#title' => t('User - Rep[lacement]Tags'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );
  
  $form['user'][] = _reptag_manage_table_builder($user->uid);
  $form['#base'] = '_reptag_manage_tags';
  return _reptag_manage_form($form, $user->uid);
}

/**
 * function _reptag_manage_table_builder().
 * (build table for tags administration)
 */
function _reptag_manage_table_builder($uid = 0) {
  // load tags table
  $reptag_table = _reptag_table_load(array($uid), 2);
  $form['#parents'] = array();
  $form['table'] = array('#theme' => 'reptag_tags_table');
  // add sample data
  if (variable_get('reptag_add_sample', 1)) {
    $form['table']['sample']['tag'] = array(
      '#value' => t('<strong>MYTAG</strong>')
    );
    $form['table']['sample']['replacement'] = array(
      '#value' => t('<strong>Hello, I\'m Mr. Repl Text.</strong>')
    );
  }
  // create tags list/table
  $num = 0;
  foreach ($reptag_table as $tag => $replacement) {
    $form['table'][$num]['tag'] = array(
      '#type' => 'textfield',
      '#default_value' => $tag,
      '#size' => 20,
      '#maxlength' => 255
    );
    $form['table'][$num]['arrow'] = array(
      '#value' => '<img src="'.drupal_get_path('module', 'reptag').'/images/arrow.png" alt="" />'
    );
    if (variable_get('reptag_textarea', 1)) {
      $form['table'][$num]['replacement'] = array(
        '#type' => 'textarea',
        '#default_value' => $replacement,
        '#rows' => 1,
        '#cols' => 60,
        '#maxlength' => 1024
      );
    }
    else {
      $form['table'][$num]['replacement'] = array(
        '#type' => 'textfield',
        '#default_value' => $replacement,
        '#size' => 65,
        '#maxlength' => 1024
      );
    }
    $form['table'][$num]['edit'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Edit'),
      '#name' => 'edit',
      '#attributes' => array('id' => $num),
      '#button_type' => 'reptag-button-edit',
    );
    $form['table'][$num]['remove'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Del'),
      '#name' => 'remove',
      '#attributes' => array('id' => $num),
      '#button_type' => 'reptag-button-remove',
    );
    $num++;
  }
  $form['table'][$num]['tag'] = array(
    '#type' => 'textfield',
    '#default_value' => '',
    '#size' => 20,
    '#maxlength' => 255,
    '#prefix' => '<div id=reptag-tag-add>',
    '#suffix' => '</div>'
  );
  $form['table'][$num]['arrow'] = array(
    '#value' => '<img src="'.drupal_get_path('module', 'reptag').'/images/arrow.png" alt="" />'
  );
  if (variable_get('reptag_textarea', 1)) {
    $form['table'][$num]['replacement'] = array(
      '#type' => 'textarea',
      '#default_value' => '',
      '#rows' => 1,
      '#cols' => 60,
      '#maxlength' => 1024
    );
  }
  else {
    $form['table'][$num]['replacement'] = array(
      '#type' => 'textfield',
      '#default_value' => '',
      '#size' => 65,
      '#maxlength' => 1024
    );
  }
  $form['table'][$num]['add'] = array(
    '#type' => 'reptag_button',
    '#value' => t('Add'),
    '#name' => 'add',
    '#attributes' => array('id' => $num),
    '#button_type' => 'reptag-button-add',
  );
  
  return $form;
}

/**
 * function _reptag_manage_tags_submit().
 */
function _reptag_manage_tags_submit($form_id, $form_data) {
  if ($form_data['op'] == t('Save configuration')) {
    // save configuration
    _reptag_table_clear($form_data['uid']);
    foreach ($form_data['table'] as $reptag) {
      if (!empty($reptag['tag']) && !empty($reptag['replacement'])) { 
        _reptag_table_set('#\$'.strtoupper($reptag['tag']).'\$#si', $reptag['replacement'], $form_data['uid']);
      }
    }
  }
  else {
    // reset configuration
    _reptag_table_clear($form_data['uid']);
  }
  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * function _reptag_manage_admin_general_submit().
 */
function _reptag_manage_admin_general_submit($form_id, $form_data) {
  if ($form_data['op'] == t('Save configuration')) {
    // save general configuration
    variable_set('reptag_enable', $form_data['reptag_enable']);
    variable_set('reptag_preview', $form_data['reptag_preview']);
    variable_set('reptag_showhelp', $form_data['reptag_showhelp']);
    variable_set('reptag_tableprio', $form_data['reptag_tableprio']);
    // expert
    variable_set('reptag_validate', $form_data['reptag_validate']);
    if (!variable_get('reptag_cache', 0) && $form_data['reptag_cache']) {
      drupal_set_message(t('You just enabled \'Partial PageCache\'. Please visit !maintenance to migrate your existing nodes.', array('!maintenance' => l(t('Maintenance'), 'admin/settings/reptag/maintenance'))), 'error');
    }
    else if (variable_get('reptag_cache', 0) && !$form_data['reptag_cache']) {
      drupal_set_message(t('You just disabled \'Partial PageCache\'. Please visit !maintenance to restore your nodes.', array('!maintenance' => l(t('Maintenance'), 'admin/settings/reptag/maintenance'))), 'error');
    }
    variable_set('reptag_cache', $form_data['reptag_cache']);
    variable_set('reptag_staticonly', $form_data['reptag_staticonly']);
    variable_set('reptag_debug', $form_data['reptag_debug']);
    // ui
    variable_set('reptag_add_sample', $form_data['reptag_add_sample']);
    variable_set('reptag_javascript', $form_data['reptag_javascript']);
    variable_set('reptag_textarea', $form_data['reptag_textarea']);
    // integration
    variable_set('reptag_quicktags', $form_data['reptag_quicktags']);
    variable_set('reptag_workspace', $form_data['reptag_workspace']);
    // node types
    foreach ($form_data['node_types'] as $type => $value) {
      variable_set('reptag_type_disable_'.$type, $value);
    }
  }
  else {
    // reset configuration
    variable_set('reptag_enable', 1);
    variable_set('reptag_preview', 1);
    variable_set('reptag_showhelp', 1);
    variable_set('reptag_tableprio', 0);
    // expert
    variable_set('reptag_validate', 1);
    variable_set('reptag_cache', 0);
    variable_set('reptag_staticonly', 0);
    variable_set('reptag_debug', 0);
    // ui
    variable_set('reptag_add_sample', 1);
    variable_set('reptag_javascript', 1);
    variable_set('reptag_textarea', 1);
    // integration
    variable_set('reptag_quicktags', 0);
    variable_set('reptag_workspace', 0);
  }
  if (variable_get('reptag_validate', 1) && !class_exists('DOMDocument') && !function_exists('xml_parser_create')) {
    drupal_set_message(t('You dont have neither \'DOMDocument\' class nor \'xml_parser_create\' function available.<br />HTML-Validation is not possible and will be disabled!'), 'error');
    variable_set('reptag_validate', 0);
  }
  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * function _reptag_manage_admin_modules_submit().
 */
function _reptag_manage_admin_modules_submit($form_id, $form_data) {
  if ($form_data['op'] == t('Save configuration')) {
    // save modules configuration
    foreach ($form_data['modules']['table'] as $module => $value) {
      variable_set('reptag_module_'.$module, $value['enabled']);
      // check prerequisites
      if ($value['enabled'] && function_exists('_'.$module.'_require')) {
        $require_func = '_'.$module.'_require';
        if (!$require_func()) {
          drupal_set_message(t('!module module has been disabled!<br />It does not comply with all requirements (e.g. include dependencies, PEAR packages, ...).', array('!module' => strtoupper($module))), 'error');
          variable_set('reptag_module_'.$module, 0);      
        }
      }
    }
  }
  else {
    // reset configuration
    foreach ($form_data['modules']['table'] as $module => $value) {
      variable_set('reptag_module_'.$module, 0);
    }
  }
  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * function _reptag_manage_admin_roles_submit().
 */
function _reptag_manage_admin_roles_submit($form_id, $form_data) {
  if ($form_data['op'] == t('Save configuration')) {
    // save roles configuration
    foreach ($form_data['roles']['table'] as $module => $role) {
      $roles = array();
      foreach ($role as $rid => $value) {
        if ($value) {
          $roles[] = $rid;
        }
      }
      variable_set('reptag_module_'.$module.'_roles', serialize($roles));
    }
    // exclude
    foreach ($form_data['roles']['exclude']['table'] as $rid => $data) {
      if (is_array($data)) {
        _reptag_exclude_storetags_string($rid, $data['tags']);
      }
      else {
        _reptag_exclude_storetags_string($rid, $data);
      }
    }
  }
  else {
    // reset configuration
    foreach ($form_data['roles']['table'] as $module => $role) {
      variable_set('reptag_module_'.$module.'_roles', serialize(array()));
    }
    // exclude
    foreach ($form_data['roles']['exclude']['table'] as $rid => $data) {
      _reptag_exclude_storetags($rid, array());
    }
  }
  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * function theme_reptag_tags_table().
 */
function theme_reptag_tags_table($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = drupal_render($form[$key]['tag']);
    $row[] = array('data' => drupal_render($form[$key]['arrow']), 'id' => 'arrow');
    $row[] = drupal_render($form[$key]['replacement']);
    if (isset($form[$key]['add'])) {
      $row[] = drupal_render($form[$key]['add']);
    }
    else {
      $row[] = drupal_render($form[$key]['edit']);
    }
    $row[] = drupal_render($form[$key]['remove']);
    $rows[] = array('data' => $row, 'id' => 'tag-'.$key);
  }
  
  $header = array(t('Tag'), '', t('Replacement'));
  return theme('table', $header, $rows); 
}

/**
 * function theme_reptag_modules_table().
 */
function theme_reptag_modules_table($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = drupal_render($form[$key]['enabled']);
    $row[] = drupal_render($form[$key]['module']);
    $row[] = drupal_render($form[$key]['info']);
    $row[] = drupal_render($form[$key]['static']);
    $rows[] = $row;
  }
  
  $header = array(t('Enabled'), t('Module'), t('Info'), t('Static'));
  return theme('table', $header, $rows);
}

/**
 * function theme_reptag_roles_table().
 */
function theme_reptag_roles_table($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    foreach ($form[$key] as $item => $data) {
      if($item == 'label' || is_numeric($item)) {
        $row[] = drupal_render($data);
      }
    }
    $rows[] = $row;
  }
  
  $header = $form['header']['#data'];
  return theme('table', $header, $rows);
}

/**
 * function theme_reptag_exclude().
 */
function theme_reptag_exclude($form) {
  $rows[] = array(array('data' => t('<em>Enter tags to exclude seperated by \'|\', e.g. \'$MYTAG$\' or \'#{BACK}(.*?){/BACK}#s\'.</em>'), 'colspan' => 4));
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = drupal_render($form[$key]['role']);
    $row[] = drupal_render($form[$key]['tags']);
    $row[] = drupal_render($form[$key]['append']);
    $row[] = drupal_render($form[$key]['remove']);
    $rows[] = array('data' => $row, 'id' => 'role-'.$key);
  }
  
  $header = array(t('Exclude for role'), t('Tags'), '', '');
  return theme('table', $header, $rows);
}

/**
 * function theme_reptag_exclude_add().
 */
function theme_reptag_exclude_add($form) {
  $row[] = drupal_render($form['value']);
  $row[] = drupal_render($form['button']);
  $row[] = array('data' => '', 'id' => 'reptag-exclude-wait');
  $rows[] = $row;
  
  $header = array(t('Enter tag to exclude:'), '', '');
  return '<div id="reptag-exclude-add" style="display:none;">'.theme('table', $header, $rows).'</div>';
}

/**
 * function _reptag_manage_form().
 * (add hidden fields for ajax support and default buttons) 
 */
function _reptag_manage_form($form, $uid = 0) {
  if (variable_get('reptag_javascript', 1)) {
    drupal_add_js(drupal_get_path('module', 'reptag').'/reptag_helper.js');
    drupal_add_js('var reptag_id=\'\';', 'inline');
    drupal_add_js('var reptag_uid=\''.$uid.'\';', 'inline');
    drupal_add_js('var reptag_ajax_cbpath=\''.check_url(url('reptag/ajaxcb', NULL, NULL, TRUE)).'\';', 'inline');
    drupal_add_js('var reptag_ajax_img=\''.drupal_get_path('module', 'reptag').'/images/\';', 'inline');
  }
  
  $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));
  $form['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset to defaults'));
  return $form;
}

/**
 * function theme_reptag_button().
 */
function theme_reptag_button($element) {
  $element['#attributes']['disabled'] = 'disabled';
  return '<input type="button" class="form-'. $element['#button_type'] .'" name="'. $element['#name'] .'" value="'. check_plain($element['#value']) .'" '. drupal_attributes($element['#attributes']) ." />\n";
}

?>
