<?php
// $Id: reptag_maintenance.inc,v 1.8 2006/10/20 07:44:24 profix898 Exp $

/**
 * function _reptag_maintenance_form().
 * (maintenance options, mainly related to partial cache feature)
 */
function _reptag_maintenance_form() {
  $stored = db_fetch_object(db_query("SELECT COUNT(*) AS rows FROM {reptag_storage}"));
  $form['maintenance'] = array(
    '#type' => 'fieldset',
    '#title' => t('RepTag - Maintenance'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE
  ); 
  $form['maintenance']['reptag_maintenance_migrate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Migrate existing nodes to reptag storage'),
    '#return_value' => 1,
    '#default_value' => FALSE,
    '#description' => t('Use this option after you enabled \'PartialCache\' to create a cacheable version of every node.'),
    '#disabled' => (variable_get('reptag_cache', 0) && !$stored->rows) ? FALSE : TRUE
  );
  $form['maintenance']['reptag_maintenance_regenerate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Regenerate all cached pages'),
    '#return_value' => 1,
    '#default_value' => FALSE,
    '#description' => t('Use this option to update all cached pages. Reptag will run again on the original node and create a new cacheable version.'),
    '#disabled' => (variable_get('reptag_cache', 0) && $stored->rows) ? FALSE : TRUE
  );
  $form['maintenance']['reptag_maintenance_restore'] = array(
    '#type' => 'checkbox',
    '#title' => t('Restore nodes from reptag storage'),
    '#return_value' => 1,
    '#default_value' => FALSE,
    '#description' => t('Use this option after you disabled \'PartialCache\' to remove the cached pages and restore the original nodes.'),
    '#disabled' => $stored->rows ? FALSE : TRUE
  );
  $form['maintenance']['interval'] = array('#theme' => 'reptag_maintenance_interval');
  $form['maintenance']['interval']['pretitle'] = array('#value' => t('Process'));
  $form['maintenance']['interval']['posttitle'] = array('#value' => t('per step'));
  $form['maintenance']['interval']['reptag_interval'] = array(
    '#type' => 'select',
    '#title' => '',
    '#default_value' => variable_get('reptag_interval', 20),
    '#options' => array(
      20 => t('!n nodes', array('!n' => 20)),
      50 => t('!n nodes', array('!n' => 50)),
      100 => t('!n nodes', array('!n' => 100)),
      500 => t('!n nodes', array('!n' => 500)),
      1000 => t('!n nodes', array('!n' => 1000)),
      5000 => t('!n nodes', array('!n' => 5000))),
    '#disabled' => variable_get('reptag_cache', 0) ? FALSE : TRUE
  );
  
  $form['maintenance']['info'] = array(
    '#value' => t('Warning: Be careful using these options. There will be no backup of your database tables.'),
    '#prefix' => '<p><strong>',
    '#suffix' => '</strong></p>'
  );
  
  $form['buttons']['perform'] = array('#type' => 'submit', '#value' => t('Perform'));
  $form['#submit']['_reptag_maintenance_submit'] = array();
  return $form;
}

/**
 * function theme_reptag_maintenance_interval().
 * (theme maintenance interval form) 
 */
function theme_reptag_maintenance_interval($form) {
  $rows[] = array(
    drupal_render($form['pretitle']),
    drupal_render($form['reptag_interval']),
    drupal_render($form['posttitle'])
  );
  
  return theme('table', array(), $rows);
}

/**
 * function _reptag_maintenance_submit().
 */
function _reptag_maintenance_submit($form_id, $form_data) {
  if ($form_data['op'] == t('Perform')) {
    $interval = $form_data['reptag_interval'];
    variable_set('reptag_interval', is_numeric($interval) ? $interval : variable_get('reptag_interval', 20));
  
    if (($form_data['reptag_maintenance_migrate']
      + $form_data['reptag_maintenance_regenerate']
      + $form_data['reptag_maintenance_restore']) > 1
      ) {
      drupal_set_message(t('More than one option selected.'), 'error');
      return;
    }
    
    if ($form_data['reptag_maintenance_migrate']) {
      _reptag_maintenance_start('migrate');
    }
    else if ($form_data['reptag_maintenance_regenerate']) {
      _reptag_maintenance_start('regenerate');
    }
    else if ($form_data['reptag_maintenance_restore']) {
      _reptag_maintenance_start('restore');
    }
    else {
      drupal_set_message(t('No option selected.'), 'error');
    }
  }
}

/**
 * function _reptag_maintenance_migrate().
 * (migrate existing nodes to reptag storage)  
 */
function _reptag_maintenance_migrate(&$position) {
  require_once (drupal_get_path('module', 'reptag').'/reptag_process.inc');
  
  $total = db_fetch_object(db_query("SELECT COUNT(*) AS rows FROM {node_revisions}"));
  if (!$total->rows) {
    return 100;
  }
  
  $interval = variable_get('reptag_interval', 20);
  $result = db_query_range("SELECT * FROM {node_revisions}", $position, $interval);
  while ($node = db_fetch_object($result)) {
    // debug
    if (variable_get('reptag_debug', 0)) {
      drupal_set_message('migrate: nid '.$node->nid.', vid '.$node->vid, 'error');
    }
    _reptag_storage_setpage($node);
    reptag_process($node, REPTAG_NODETEASER, REPTAG_PROCCACHE);
    reptag_process($node, REPTAG_NODEFULL, REPTAG_PROCCACHE);
    _reptag_node_update($node);
  }
  $position += $interval;
  
  return ( 100 * $position ) / $total->rows;
}

/**
 * function _reptag_maintenance_regenerate().
 * (regenerate nodes from reptag storage) 
 */
function _reptag_maintenance_regenerate(&$position) {
  require_once (drupal_get_path('module', 'reptag').'/reptag_process.inc');
  
  $total = db_fetch_object(db_query("SELECT COUNT(*) AS rows FROM {reptag_storage}"));
  if (!$total->rows) {
    return 100;
  }
  
  $interval = variable_get('reptag_interval', 20);
  $result = db_query_range("SELECT * FROM {reptag_storage}", $position, $interval);
  while ($node = db_fetch_object($result)) {
    if (variable_get('reptag_debug', 0)) {
      drupal_set_message('regenerate: nid '.$node->nid.', vid '.$node->vid, 'error');
    }
    reptag_process($node, REPTAG_NODETEASER, REPTAG_PROCCACHE);
    reptag_process($node, REPTAG_NODEFULL, REPTAG_PROCCACHE);
    _reptag_node_update($node);
  }
  $position += $interval;
  
  return ( 100 * $position ) / $total->rows;
}

/**
 * function _reptag_maintenance_restore().
 * (restore nodes from reptag storage) 
 */
function _reptag_maintenance_restore(&$position) {
  require_once (drupal_get_path('module', 'reptag').'/regtag_helper.inc');
  
  $total = db_fetch_object(db_query("SELECT COUNT(*) AS rows FROM {reptag_storage}"));
  if (!$total->rows) {
    return 100;
  }
  
  $interval = variable_get('reptag_interval', 20);
  $result = db_query_range("SELECT * FROM {reptag_storage}", $position, $interval);
  while ($node = db_fetch_object($result)) {
    if (variable_get('reptag_debug', 0)) {
      drupal_set_message('restore: nid '.$node->nid.', vid '.$node->vid, 'error');
    }
    _reptag_node_update($node);
  }
  $position += $interval;
  if ($position >= $total->rows) {
    db_query("TRUNCATE TABLE {reptag_storage}");
  }
  
  return ( 100 * $position ) / $total->rows;
}

/**
 * function _reptag_maintenance_start().
 */
function _reptag_maintenance_start($mode) {
  $_SESSION['reptag_progress_mode'] = $mode;
  $_SESSION['reptag_progress_position'] = 0;
  drupal_goto('admin/settings/reptag/maintenance_progress');
}

/**
 * function _reptag_maintenance_progress().
 * (maintenance progress page) 
 */
function _reptag_maintenance_progress() {
  $mode = $_SESSION['reptag_progress_mode'];
  $position = $_SESSION['reptag_progress_position'];
  if ($_SERVER['REQUEST_METHOD'] == 'GET') {
    switch ($mode) {
      case 'migrate':
        $msg = t('Migrating existing nodes ...');
        $percent = _reptag_maintenance_migrate($position);
        break;
      case 'regenerate':
        $msg = t('Regenerating cached pages ...');
        $percent = _reptag_maintenance_regenerate($position);
        break;
      case 'restore':
        $msg = t('Restoring node revisions ...');
        $percent = _reptag_maintenance_restore($position);
        break;
      default:
        $msg = t('Running maintenance ...');
        (int)fmod($position, 100.0);
    }
    if ($percent >= 100) {
      unset($_SESSION['reptag_progress_mode']);
      unset($_SESSION['reptag_progress_position']);
      drupal_set_message(t('Maintenance operation successfully comleted.'));
      drupal_goto('admin/settings/reptag/maintenance');
      return;
    }
    $_SESSION['reptag_progress_position'] = $position;
  }
  
  drupal_set_title('Rep[lacement]Tags Maintenance');
  drupal_set_html_head('<meta http-equiv="Refresh" content="0; URL=index.php?q=admin/settings/reptag/maintenance_progress">');
  print theme('reptag_maintenance_page', theme('progress_bar', round($percent, 1), $msg), FALSE);
  exit();
}

function theme_reptag_maintenance_page($content, $messages = TRUE, $partial = FALSE) {
  drupal_set_header('Content-Type: text/html; charset=utf-8');
  drupal_set_html_head('<link rel="shortcut icon" href="'. base_path() .'misc/favicon.ico" type="image/x-icon" />');
  
  $css = array('module' => array());
  $css['module']['misc/maintenance.css'] = array('path' => 'misc/maintenance.css', 'media' => 'all');
  $css['module']['modules/system/system.css'] = array('path' => 'modules/system/system.css', 'media' => 'all');

  $output = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
  $output .= '<html xmlns="http://www.w3.org/1999/xhtml">';
  $output .= '<head>';
  $output .= ' <title>'. strip_tags(drupal_get_title()) .'</title>';
  $output .= drupal_get_html_head();
  $output .= drupal_get_css($css);
  $output .= drupal_get_js();
  $output .= '</head>';
  $output .= '<body>';
  $output .= '<h2>' . drupal_get_title() . '</h2>';
  
  $output .= "\n<!-- begin content -->\n";
  $output .= $content;
  $output .= "\n<!-- end content -->\n";
  
  $output .= '</body></html>';

  return $output;
}

?>
