<?php
// $Id: reptag_process.inc,v 1.21 2006/10/18 09:59:37 profix898 Exp $

require_once (drupal_get_path('module', 'reptag').'/regtag_helper.inc');

define('REPTAG_PROCALL',      0);
define('REPTAG_PROCCACHE',    1);
define('REPTAG_PROCDYNAMIC',  2);

define('REPTAG_NODEFULL',     1);
define('REPTAG_NODETEASER',   2);
define('REPTAG_COMMENT',      3);
define('REPTAG_USERTEXT',     4);

/**
 * Perform the actual reptag processing
 * (This function does all the filter/replacement stuff)
 *
 * @param $content
 *   An array containing the content to process.    
 * @param $type
 *   An integer to set the content type (e.g. REPTAG_NODETEASER, ...).
 * @param $proc
 *   An integer to set the processing mode (e.g. REPTAG_PROCALL, ...).
 *
 *   See DEVELOPER.TXT for detailed information  
 */
function reptag_process(&$content, $type = 0, $proc = REPTAG_PROCESSALL) {  
  // select text to filter
  switch ($type) {
    case REPTAG_NODEFULL:
      $text = &$content->body;
      break;
    case REPTAG_NODETEASER:
      $text = &$content->teaser;
      break;
    case REPTAG_COMMENT:
      $text = &$content->comment;
      break;
    case REPTAG_USERTEXT:
      $text = &$content->text;
      break;
    default:
      return;
  }
    
  // include .tags modules ...
  $reptag_modules = array();
  $files = file_scan_directory(drupal_get_path('module', 'reptag'), '\.tags$');
  foreach ($files as $filename => $file) {
    if (variable_get('reptag_module_'.$file->name, 1)) {
      include_once $file->filename;
      // ... depending on processing mode
      switch ($proc) {
        case REPTAG_PROCALL:
          $reptag_modules[] = $file->name;
          break;
        case REPTAG_PROCCACHE:
          if (is_array($reptags_info[$file->name]) && $reptags_info[$file->name][1]) {
            $reptag_modules[] = $file->name;
          }
          break;
        case REPTAG_PROCDYNAMIC:
          if (!is_array($reptags_info[$file->name])) {
            $reptag_modules[] = $file->name;
          }
          else if ($reptags_info[$file->name][1]) {
            $reptag_modules[] = $file->name;
          }
          break;
      }
    }
  }
  
  if ($proc == REPTAG_PROCDYNAMIC && empty($reptag_modules)) {
    return;
  }
  
  // roles associated with current node
  $content_roles = _reptag_user_get_roles($content->uid);

  // load exclude tags
  $exclude_tags = array();
  foreach ($content_roles as $rid) {
    $rid_tags = _reptag_exclude_loadtags($rid);
    $exclude_tags = array_merge($exclude_tags, $rid_tags);
  }
  
  // load side-wide and user tags from tables
  if ($proc != REPTAG_PROCDYNAMIC) {
  
    $iid = array(0, $content->uid);
    $reptag_table = _reptag_table_load($iid);
    
    // remove all html tags for plain text roles
    $plainrep_roles = unserialize(variable_get('reptag_module_plainrep_roles', serialize(array())));
    if (array_intersect($plainrep_roles, $content_roles)) {
      foreach ($reptag_table as $tag => $repl) {
        $reptag_table[$tag] = check_plain(preg_replace('#<[\/\!]*?[^<>]*?>#si', '', $repl));
      }
    }
    
    // filter exclude_tags from reptag_table
    if (!empty($exclude_tags)) {
      foreach ($exclude_tags as $key) {
        $key = '#\\'.strtoupper(substr($key, 0, -1)).'\$#si'; 
        unset($reptag_table[$key]);
      }
    }
    
  }
  
  // load module variables (according to permissions)
  if (!empty($reptag_modules)) {
    foreach ($reptag_modules as $module) {
      $module_roles = unserialize(variable_get('reptag_module_'.$module.'_roles', serialize(array())));
      if (function_exists('_'.$module.'_init') && (array_intersect($module_roles, $content_roles) || $content->uid == 1)) {
        $init_func = '_'.$module.'_init';
        if (!$reptags[$module] = $init_func($content)) {
          unset($reptags[$module]);
        }
        else {
          if (function_exists('_'.$module.'_require')) {
            $require_func = '_'.$module.'_require';
            if (!$require_func()) {
              unset($reptags[$module]);
            }
          }
        }
      }
    }
  }
  
  // filter exclude_tags from reptags
  $reptags = _reptag_filter_exclude($reptags, $exclude_tags);
  
  // debug
  if (variable_get('reptag_debug', 0)) {
    drupal_set_message('reptags: '.nl2br(htmlspecialchars(print_r($reptags, true))), 'error');
    drupal_set_message('reptag_table: '.nl2br(htmlspecialchars(print_r($reptag_table, true))), 'error');
    drupal_set_message('node: '.nl2br(htmlspecialchars(print_r($content, true))), 'error');
  }
  
  // process table tags
  if ($proc != REPTAG_PROCDYNAMIC) {
    $text = preg_replace(array_keys($reptag_table), array_values($reptag_table), $text);
  }
  
  // process module tags
  if (!empty($reptags)) {
    foreach ($reptags as $module => $tags) {
      if (function_exists('_'.$module.'_process')) {
        $process_func = '_'.$module.'_process';
        $text = $process_func($text, $tags, $content);
      }
      else {
        $text = preg_replace(array_keys($tags), array_values($tags), $text);
      }
    }
  }
  
  // validate on preview
  if ($content->in_preview && variable_get('reptag_validate', 1)) {
    _reptag_validate_xml($text);
  }
}

/**
 * function _reptag_filter_exclude().
 */
function _reptag_filter_exclude($reptags, $exclude_tags) {
  if (!empty($reptags) && !empty($exclude_tags)) {
    foreach ($reptags as $module => $tags) {
      foreach($tags as $tag => $repl) {
        foreach($exclude_tags as $exclude_tag) {
          if (!empty($tag) && !empty($exclude_tag)) {
            if (stristr($tag, $exclude_tag)) {
              unset($reptags[$module][$tag]);
            }
          }
        }
      }
    }
  }
  
  return $reptags; 
}

/**
 * function _reptag_validate_xml().
 * (check current content being well-formed x(ht)ml)
 */
function _reptag_validate_xml($xml) {
  $version = phpversion();
  if (version_compare($version, '4.3.0', '>=')) {
    if (version_compare($version, '5.0.0', '>=') && class_exists('DOMDocument')) {
      // PHP 5.0.0+
      
      // extract html tags
      preg_match_all('/<(\/{0,1}[\w]+)[^>]*>/', $xml, $matches);
      $matches = $matches[0];
      
$pretag = <<< PRE
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title></title>
</head>
<body>
PRE;
      
$posttag = <<< POST
</body>
</html>
POST;
      
      $xmlhtml = $pretag.implode('', $matches).$posttag;
      
      $dom = new DOMDocument;
      $status = ini_set('html_errors', FALSE);
      $dom->loadHTML($xmlhtml);
      if (!$dom->validate()) {
        ini_set('html_errors', $status);
        return FALSE;
      }
      ini_set('html_errors', $status);

    }
    else {
      // PHP 4.3.0+
    
      if (!function_exists('xml_parser_create')) {
        return TRUE;
      }
      
      $xmlhtml = '<body>'.$xml.'</body>';
      $xml_parser = xml_parser_create();      
       if (!xml_parse($xml_parser, $xmlhtml)) {
          drupal_set_message('XML error: '.xml_error_string(xml_get_error_code($xml_parser)).' at line '.xml_get_current_line_number($xml_parser), 'error');
          return FALSE;
       }
      xml_parser_free($xml_parser);
    
    }
  }

  return TRUE;
}

?>
